import { isNumber, isString, getPrefixCls, isObject } from "@gx-design-vue/pro-utils";
import { defineComponent, inject, ref, computed, onBeforeUnmount, toRef, createVNode, Transition, Fragment, watch, nextTick, provide, reactive, onMounted } from "vue";
import { useEventListener, isClient, useResizeObserver } from "@vueuse/core";
var config = "";
var style = "";
const scrollbarProps = {
  height: {
    type: [String, Number],
    default: ""
  },
  maxHeight: {
    type: [String, Number],
    default: ""
  },
  native: {
    type: Boolean,
    default: false
  },
  wrapStyle: {
    type: [String, Array],
    default: ""
  },
  wrapClass: {
    type: [String, Array],
    default: ""
  },
  viewClass: {
    type: [String, Array],
    default: ""
  },
  viewStyle: {
    type: [String, Array, Object],
    default: ""
  },
  noresize: Boolean,
  tag: {
    type: String,
    default: "div"
  },
  always: {
    type: Boolean,
    default: false
  },
  minSize: {
    type: Number,
    default: 20
  }
};
const scrollbarEmits = {
  scroll: ({ scrollTop, scrollLeft }) => isNumber(scrollTop) && isNumber(scrollLeft)
};
const scrollbarContextKey = Symbol("scrollbarContextKey");
const barProps = {
  className: String,
  always: {
    type: Boolean,
    default: true
  },
  width: {
    type: String,
    default: ""
  },
  height: {
    type: String,
    default: ""
  },
  ratioX: {
    type: Number,
    default: 1
  },
  ratioY: {
    type: Number,
    default: 1
  }
};
const thumbProps = {
  className: String,
  vertical: Boolean,
  size: String,
  move: Number,
  ratio: {
    type: Number,
    required: true
  },
  always: Boolean
};
const BAR_MAP = {
  vertical: {
    offset: "offsetHeight",
    scroll: "scrollTop",
    scrollSize: "scrollHeight",
    size: "height",
    key: "vertical",
    axis: "Y",
    client: "clientY",
    direction: "top"
  },
  horizontal: {
    offset: "offsetWidth",
    scroll: "scrollLeft",
    scrollSize: "scrollWidth",
    size: "width",
    key: "horizontal",
    axis: "X",
    client: "clientX",
    direction: "left"
  }
};
const renderThumbStyle = ({ move, size, bar }) => ({
  [bar.size]: size,
  transform: `translate${bar.axis}(${move}%)`
});
function addUnit(value, defaultUnit = "px") {
  if (!value)
    return "";
  if (isString(value)) {
    return value;
  } else if (isNumber(value)) {
    return `${value}${defaultUnit}`;
  }
  return false;
}
const COMPONENT_NAME = "Thumb";
const Thumb = defineComponent({
  props: thumbProps,
  setup(props) {
    const scrollbar = inject(scrollbarContextKey);
    if (!scrollbar)
      console.error(COMPONENT_NAME, "can not inject scrollbar context");
    const instance = ref();
    const thumb = ref();
    const thumbState = ref({});
    const visible = ref(false);
    let cursorDown = false;
    let cursorLeave = false;
    let originalOnSelectStart = isClient ? document.onselectstart : null;
    const bar = computed(() => BAR_MAP[props.vertical ? "vertical" : "horizontal"]);
    const thumbStyle = computed(() => renderThumbStyle({
      size: props.size,
      move: props.move,
      bar: bar.value
    }));
    const offsetRatio = computed(() => instance.value[bar.value.offset] ** 2 / (scrollbar == null ? void 0 : scrollbar.wrapElement)[bar.value.scrollSize] / ((props == null ? void 0 : props.ratio) || 0) / thumb.value[bar.value.offset]);
    const clickThumbHandler = (e) => {
      var _a;
      e.stopPropagation();
      if (e.ctrlKey || [1, 2].includes(e.button))
        return;
      (_a = window.getSelection()) == null ? void 0 : _a.removeAllRanges();
      startDrag(e);
      const el = e.currentTarget;
      if (!el)
        return;
      thumbState.value[bar.value.axis] = el[bar.value.offset] - (e[bar.value.client] - el.getBoundingClientRect()[bar.value.direction]);
    };
    const clickTrackHandler = (e) => {
      if (!thumb.value || !instance.value || !(scrollbar == null ? void 0 : scrollbar.wrapElement))
        return;
      const offset = Math.abs(e.target.getBoundingClientRect()[bar.value.direction] - e[bar.value.client]);
      const thumbHalf = thumb.value[bar.value.offset] / 2;
      const thumbPositionPercentage = (offset - thumbHalf) * 100 * offsetRatio.value / instance.value[bar.value.offset];
      scrollbar.wrapElement[bar.value.scroll] = thumbPositionPercentage * scrollbar.wrapElement[bar.value.scrollSize] / 100;
    };
    const startDrag = (e) => {
      e.stopImmediatePropagation();
      cursorDown = true;
      document.addEventListener("mousemove", mouseMoveDocumentHandler);
      document.addEventListener("mouseup", mouseUpDocumentHandler);
      originalOnSelectStart = document.onselectstart;
      document.onselectstart = () => false;
    };
    const mouseMoveDocumentHandler = (e) => {
      if (!instance.value || !thumb.value)
        return;
      if (cursorDown === false)
        return;
      const prevPage = thumbState.value[bar.value.axis];
      if (!prevPage)
        return;
      const offset = (instance.value.getBoundingClientRect()[bar.value.direction] - e[bar.value.client]) * -1;
      const thumbClickPosition = thumb.value[bar.value.offset] - prevPage;
      const thumbPositionPercentage = (offset - thumbClickPosition) * 100 * offsetRatio.value / instance.value[bar.value.offset];
      scrollbar.wrapElement[bar.value.scroll] = thumbPositionPercentage * scrollbar.wrapElement[bar.value.scrollSize] / 100;
    };
    const mouseUpDocumentHandler = () => {
      cursorDown = false;
      thumbState.value[bar.value.axis] = 0;
      document.removeEventListener("mousemove", mouseMoveDocumentHandler);
      document.removeEventListener("mouseup", mouseUpDocumentHandler);
      restoreOnselectstart();
      if (cursorLeave)
        visible.value = false;
    };
    const mouseMoveScrollbarHandler = () => {
      cursorLeave = false;
      visible.value = !!props.size;
    };
    const mouseLeaveScrollbarHandler = () => {
      cursorLeave = true;
      visible.value = cursorDown;
    };
    onBeforeUnmount(() => {
      restoreOnselectstart();
      document.removeEventListener("mouseup", mouseUpDocumentHandler);
    });
    const restoreOnselectstart = () => {
      if (document.onselectstart !== originalOnSelectStart)
        document.onselectstart = originalOnSelectStart;
    };
    useEventListener(toRef(scrollbar, "scrollbarElement"), "mousemove", mouseMoveScrollbarHandler);
    useEventListener(toRef(scrollbar, "scrollbarElement"), "mouseleave", mouseLeaveScrollbarHandler);
    return () => {
      return createVNode(Transition, {
        "name": `${props.className}-fade`
      }, {
        default: () => [createVNode("div", {
          "ref": (e) => instance.value = e,
          "class": props.always || visible.value ? [`${props.className}-bar`, "is-" + bar.value.key] : [`${props.className}-bar`, "is-" + bar.value.key, "hidden"],
          "onMousedown": clickTrackHandler
        }, [createVNode("div", {
          "ref": (e) => thumb.value = e,
          "class": `${props.className}-thumb`,
          "style": thumbStyle.value,
          "onMousedown": clickThumbHandler
        }, null)])]
      });
    };
  }
});
Thumb.inheritAttrs = false;
const Bar = defineComponent({
  props: barProps,
  setup(props) {
    const moveX = ref(0);
    const moveY = ref(0);
    const GAP = 4;
    const handleScroll = (wrap) => {
      if (wrap) {
        const offsetHeight = wrap.offsetHeight - GAP;
        const offsetWidth = wrap.offsetWidth - GAP;
        moveY.value = wrap.scrollTop * 100 / offsetHeight * props.ratioY;
        moveX.value = wrap.scrollLeft * 100 / offsetWidth * props.ratioX;
      }
    };
    return {
      moveX,
      moveY,
      handleScroll
    };
  },
  render() {
    return createVNode(Fragment, null, [createVNode(Thumb, {
      "className": this.className,
      "move": this.moveX,
      "ratio": this.ratioX,
      "size": this.width,
      "always": this.always
    }, null), createVNode(Thumb, {
      "className": this.className,
      "move": this.moveY,
      "ratio": this.ratioY,
      "size": this.height,
      "vertical": true,
      "always": this.always
    }, null)]);
  }
});
Bar.inheritAttrs = false;
const GScrollbars = defineComponent({
  name: "GScrollbars",
  props: scrollbarProps,
  emits: scrollbarEmits,
  setup(props, {
    emit
  }) {
    const prefixCls = getPrefixCls({
      suffixCls: "scrollbar"
    });
    let stopResizeObserver = void 0;
    let stopResizeListener = void 0;
    const scrollbar$ = ref();
    const wrap$ = ref();
    const resize$ = ref();
    const sizeWidth = ref("0");
    const sizeHeight = ref("0");
    const barRef = ref();
    const moveX = ref(0);
    const moveY = ref(0);
    const ratioY = ref(1);
    const ratioX = ref(1);
    const SCOPE = "GScrollbars";
    const GAP = 4;
    const style2 = computed(() => {
      const style3 = {};
      if (props.height)
        style3.height = addUnit(props.height);
      if (props.maxHeight)
        style3.maxHeight = addUnit(props.maxHeight);
      return [props.wrapStyle, style3];
    });
    const handleScroll = () => {
      var _a;
      if (wrap$.value) {
        (_a = barRef.value) == null ? void 0 : _a.handleScroll(wrap$.value);
        emit("scroll", {
          scrollTop: wrap$.value.scrollTop,
          scrollLeft: wrap$.value.scrollLeft
        });
      }
    };
    function scrollTo(arg1, arg2) {
      if (isObject(arg1)) {
        wrap$.value.scrollTo(arg1);
      } else if (isNumber(arg1) && isNumber(arg2)) {
        wrap$.value.scrollTo(arg1, arg2);
      }
    }
    const setScrollTop = (value) => {
      if (!isNumber(value)) {
        console.warn(SCOPE, "value \u5FC5\u987B\u4E3A\u6570\u5B57");
        return;
      }
      wrap$.value.scrollTop = value;
    };
    const setScrollLeft = (value) => {
      if (!isNumber(value)) {
        console.warn(SCOPE, "value \u5FC5\u987B\u4E3A\u6570\u5B57");
        return;
      }
      wrap$.value.scrollLeft = value;
    };
    const update = () => {
      if (!wrap$.value)
        return;
      const offsetHeight = wrap$.value.offsetHeight - GAP;
      const offsetWidth = wrap$.value.offsetWidth - GAP;
      const originalHeight = offsetHeight ** 2 / wrap$.value.scrollHeight;
      const originalWidth = offsetWidth ** 2 / wrap$.value.scrollWidth;
      const height = Math.max(originalHeight, props.minSize);
      const width = Math.max(originalWidth, props.minSize);
      ratioY.value = originalHeight / (offsetHeight - originalHeight) / (height / (offsetHeight - height));
      ratioX.value = originalWidth / (offsetWidth - originalWidth) / (width / (offsetWidth - width));
      sizeHeight.value = height + GAP < offsetHeight ? `${height}px` : "";
      sizeWidth.value = width + GAP < offsetWidth ? `${width}px` : "";
    };
    watch(() => props.noresize, (noresize) => {
      if (noresize) {
        stopResizeObserver == null ? void 0 : stopResizeObserver();
        stopResizeListener == null ? void 0 : stopResizeListener();
      } else {
        ({
          stop: stopResizeObserver
        } = useResizeObserver(resize$, update));
        stopResizeListener = useEventListener("resize", update);
      }
    }, {
      immediate: true
    });
    watch(() => [props.maxHeight, props.height], () => {
      if (!props.native)
        nextTick(() => {
          var _a;
          update();
          if (wrap$.value) {
            (_a = barRef.value) == null ? void 0 : _a.handleScroll(wrap$.value);
          }
        });
    });
    provide(scrollbarContextKey, reactive({
      scrollbarElement: scrollbar$,
      wrapElement: wrap$
    }));
    onMounted(() => {
      if (!props.native)
        nextTick(() => update());
    });
    return {
      barRef,
      wrap$,
      scrollbar$,
      resize$,
      style: style2,
      moveX,
      moveY,
      prefixCls,
      sizeHeight,
      sizeWidth,
      ratioX,
      ratioY,
      handleScroll,
      scrollTo,
      setScrollTop,
      setScrollLeft
    };
  },
  render() {
    const CustomTag = this.tag;
    return createVNode("div", {
      "ref": (e) => this.scrollbar$ = e,
      "class": this.prefixCls
    }, [createVNode("div", {
      "ref": (e) => this.wrap$ = e,
      "class": [this.wrapClass, `${this.prefixCls}-wrap`, this.native ? "" : `${this.prefixCls}-wrap-hidden-default`],
      "style": this.style,
      "onScroll": this.handleScroll
    }, [createVNode(CustomTag, {
      "ref": (e) => this.resize$ = e,
      "class": [`${this.prefixCls}-view`, this.viewClass],
      "style": this.viewStyle
    }, {
      default: () => {
        var _a, _b;
        return [(_b = (_a = this.$slots).default) == null ? void 0 : _b.call(_a)];
      }
    })]), !this.native && createVNode(Bar, {
      "ref": (e) => this.barRef = e,
      "className": this.prefixCls,
      "height": this.sizeHeight,
      "width": this.sizeWidth,
      "always": this.always,
      "ratioX": this.ratioX,
      "ratioY": this.ratioY
    }, null)]);
  }
});
GScrollbars.inheritAttrs = false;
export { GScrollbars, GScrollbars as default, scrollbarProps };
